class Wip():
    def print_it(self):
        print("itsa me mario")


def wet():
    print("beep boop")


def wat():
    "WEEE"
    print("testing, 1, 2, 3")


def wut(text, woop=False):
    """ Super docstring test

    Args:
        text (str): The text to print
        woop (boolean, optional): Default false
    """
    print(text)
    print(woop)


def _hidden():
    print("secret")



#!/usr/bin/env python
import sys
import argparse
import inspect
import pydoc
from typing import Any, Dict, List, Tuple, Callable, Optional
from pathlib import Path
from types import ModuleType
from collections.abc import ItemsView


def filter_vars(imported_vars: ItemsView) -> Dict[str, Callable]:
    functions = {}
    for name, obj in imported_vars:
        if callable(obj) and not name.startswith('_'):
            if inspect.isclass(obj):
                methods = inspect.getmembers(obj(), predicate=inspect.ismethod)
                for name, method in methods:
                    if not name.startswith('_'):
                        functions[obj.__name__ + "." + name] = method
            else:
                functions[obj.__name__] = obj
    return functions


def load_runfile(runfile: Path) -> ItemsView:
    importer = __import__
    directory, runfile = runfile.parent, Path(runfile.name)

    sys.path.insert(0, str(directory))
    imported: ModuleType = importer(runfile.stem)
    del sys.path[0]
    imported_vars = vars(imported).items()
    return imported_vars


def _escape_split(sep: str, argstr: str) -> str:
    escaped_sep = r'\%s' % sep

    if escaped_sep not in argstr:
        return argstr.split(sep)

    before, _, after = argstr.partition(escaped_sep)
    startlist = before.split(sep)
    unfinished = startlist[-1]
    startlist = startlist[:-1]
    endlist = _escape_split(sep, after)
    unfinished += sep + endlist[0]
    return startlist + [unfinished] + endlist[1:]


def parse_args(cmd: str) -> Tuple[str, List[str], Dict[str, str]]:
    args = []
    kwargs = {}
    if ':' in cmd:
        cmd, argstr = cmd.split(':', 1)
        for pair in _escape_split(',', argstr):
            result = _escape_split('=', pair)
            if len(result) > 1:
                k, v = result
                kwargs[k] = v
            else:
                args.append(result[0])
    return cmd, args, kwargs


def get_docstring(function: Callable, abbrv=False) -> Optional[str]:
    try:
        doc = inspect.getdoc(function)
        if abbrv:
            doc = doc.splitlines()[0].strip()
    except:
        doc = ""
    return doc


def get_function(functions: Dict[str, Callable], function_name: str) -> Optional[Callable]:
    try:
        return functions[function_name]
    except KeyError:
        print("No function named '{}' found!".format(function_name))
        return None


def print_functions(functions: Dict[str, Callable]) -> None:
    print("Available functions:")
    for fname, function in functions.items():
        doc = get_docstring(function, abbrv=True)
        print(fname + "\t" + doc)


def print_function(functions: Dict[str, Callable], function: str) -> None:
    func = get_function(functions, function)
    if func:
        print(pydoc.plain(pydoc.render_doc(
            func,
            "Displaying docstring for %s")
        ))


def run_function(functions: Dict[str, Callable], cmd: str) -> None:
    function, args, kwargs = parse_args(cmd)
    try:
        func = get_function(functions, function)
        if func:
            func(*args, **kwargs)
    except TypeError as e:
        print(e.args[0])


def main(*argv: Tuple[Any]) -> None:

    parser = argparse.ArgumentParser(description='Run functions in a file.')
    parser.add_argument('runfile', help='file containing the functions')
    parser.add_argument('function', nargs='?', help='function to run')
    parser.add_argument(
        '-l', '--list',
        action='store_true',
        help='list available functions in file'
    )
    parser.add_argument(
        '-d', '--detail',
        help='print function docstring'
    )
    args = parser.parse_args(*argv)

    runfile = Path(args.runfile).resolve()

    if not runfile.is_file:
        print("No such file '{}'".format(args.runfile))
        sys.exit(1)

    imported_vars = load_runfile(runfile)
    functions = filter_vars(imported_vars)

    if args.list:
        print_functions(functions)
        sys.exit(0)

    if args.detail:
        print_function(functions, args.detail)
        sys.exit(0)

    if args.function is None:
        print("No function was selected!")
        sys.exit(1)
    run_function(functions, args.function)


if __name__ == "__main__":
    main(sys.argv[1:])



from runp import runp

import os
import io
import sys
import unittest
from pathlib import Path


class RunPTestCase(unittest.TestCase):
    def setUp(self):
        self.test_path = Path(__file__).parent
        self.runfile = self.test_path.joinpath("testfile.py")
        self.imported_vars = runp.load_runfile(self.runfile)
        self.functions = runp.filter_vars(self.imported_vars)
        self.org_stdout, sys.stdout = sys.stdout, io.StringIO()
        
    def tearDown(self) -> None:
        sys.stdout = self.org_stdout

    def test_load_runfile(self):
        self.assertTrue(len(self.imported_vars) >= len(self.functions))

    def test_filter_vars(self):
        self.assertEquals(len(self.functions), 4)

    def test_print_functions(self):
        out = """Available functions:
Wip.print_it\t
wet\t
wat\tWEEE
wut\tSuper docstring test"""
        runp.print_functions(self.functions)
        output = sys.stdout.getvalue().strip()
        self.assertEquals(str(output), out)

    def test_print_function_no_docstring(self):
        out = """Displaying docstring for function wet in module testfile

wet()"""
        runp.print_function(self.functions, "wet")
        output = sys.stdout.getvalue().strip()
        self.assertEquals(str(output), out)

    def test_print_function_multi_docstring(self):
        out = """Displaying docstring for function wut in module testfile

wut(text, woop=False)
    Super docstring test
    
    Args:
        text (str): The text to print
        woop (boolean, optional): Default false"""
        runp.print_function(self.functions, "wut")
        output = sys.stdout.getvalue().strip()
        self.assertEquals(str(output), out)

    def test_run_function_noargs(self):
        out = "testing, 1, 2, 3"
        runp.run_function(self.functions, "wat")
        output = sys.stdout.getvalue().strip()
        self.assertEquals(str(output), out)

    def test_run_function_args(self):
        out = "mytext\ndoobey"
        runp.run_function(self.functions, "wut:mytext,doobey")
        output = sys.stdout.getvalue().strip()
        self.assertEquals(str(output), out)

    def test_run_function_named_args(self):
        out = "mytext\nTrue"
        runp.run_function(self.functions, "wut:mytext,woop=True")
        output = sys.stdout.getvalue().strip()
        self.assertEquals(str(output), out)

    def test_run_function_reverse_args(self):
        out = "mytext\nTrue"
        runp.run_function(self.functions, "wut:woop=True,mytext")
        output = sys.stdout.getvalue().strip()
        self.assertEquals(str(output), out)

    def test_run_function_wrong_args(self):
        out = "wut() missing 1 required positional argument: 'text'"
        runp.run_function(self.functions, "wut")
        output = sys.stdout.getvalue().strip()
        self.assertEquals(str(output), out)

    def test_get_function_nonexistant(self):
        nofunc = "wutwut"
        out = "No function named '{}' found!".format(nofunc)
        runp.get_function(self.functions, nofunc)
        output = sys.stdout.getvalue().strip()
        self.assertEquals(str(output), out)

    def test_parse_args_noargs(self):
        inputstr = "wut"
        cmd, args, kwargs = runp.parse_args(inputstr)
        tup = (cmd, args, kwargs)
        self.assertEquals(tup, ("wut", [], {}))

    def test_parse_args_nokwargs(self):
        inputstr = "wut:wow,such,good"
        cmd, args, kwargs = runp.parse_args(inputstr)
        tup = (cmd, args, kwargs)
        self.assertEquals(tup, ("wut", ["wow", "such", "good"], {}))

    def test_parse_args(self):
        inputstr = "wut:arg=wow,'such spaces',arg2=good"
        cmd, args, kwargs = runp.parse_args(inputstr)
        tup = (cmd, args, kwargs)
        self.assertEquals(
            tup,
            ("wut", ["'such spaces'"], {"arg": "wow", "arg2": "good"})
        )

    def test_escape_split_comma(self):
        inputstr = "wut:arg=wow,'such spaces',arg2=good"
        splitted = ['wut:arg=wow', "'such spaces'", 'arg2=good']
        self.assertEquals(runp._escape_split(',', inputstr), splitted)

    def test_escape_split_equals(self):
        inputstrs = ['wut:arg=wow', "'such spaces'", 'arg2=good']
        results = [['wut:arg', 'wow'], ["'such spaces'"], ['arg2', 'good']]
        for i, inputstr in enumerate(inputstrs):
            self.assertEquals(runp._escape_split('=', inputstr), results[i])

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

from setuptools import setup, find_packages

with open('README.md') as f:
    readme = f.read()

long_description = readme

setup(
    name='runp3',
    description='runp exports Python functions from files to the command line',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Takashi Kusachi',
    author_email='aisiars@gmail.com',
    url='https://github.com/TakashiKusachi/runp',
    packages=find_packages(),
    test_suite='tests',
    install_requires=[],
    entry_points={
        'console_scripts': [
            'runp = runp.runp:main',
        ]
    },
    use_scm_version=True,
    setup_requires=[
        "setuptools_scm"
    ],
    classifiers=[
        'Environment :: Console',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Topic :: Software Development'
    ],
    python_requires='>3.7.11',
)
